/*
  File: DisplayBuffer.java
*/

package buffer;

import support.*;

public class DisplayBuffer implements Buffer
{

	private BufferPanel view_;

	private BoundedBuffer buffer_;

	private Semaphore full_;  // counts number of items

	private Semaphore empty_; // counts number of spaces

	private int size_;

	public DisplayBuffer( int size )
	{
		size_ = size;
		view_ = new BufferPanel( "Buffer", size );
		buffer_ = new BoundedBuffer( size );
		full_ = new Semaphore( 0 );
		empty_ = new Semaphore( size_ );
	}

	public void put( Object o ) throws InterruptedException
	{
		empty_.down();
		buffer_.put( o );
		updateView();
		full_.up();
	}

	public Object get() throws InterruptedException
	{
		full_.down();
		Object o = buffer_.get();
		updateView();
		empty_.up();
		return o;
	}

	public void init()
	{
		full_ = new Semaphore( 0 );
		empty_ = new Semaphore( size_ );
		buffer_.init();
		updateView();
	}

	private void updateView()
	{
		view_.setValue( buffer_.getCharBuffer(), 
						buffer_.getIn(), 
						buffer_.getOut() );
	}

	BufferPanel getView()
	{
		return view_;
	}

	Buffer getBuffer()
	{
		return buffer_;
	}

}
