/*
  File: RotatorPanel.java
*/

package support;

import java.awt.*;
import java.awt.event.*;
import support.*;

public class RotatingPanel extends Panel
                           implements Runnable
{

    ArcPanel arcPanel_;

    Button run_;

    Button pause_;

    Scrollbar slider_;

    static Color runcolor_ = Color.green;

    static Color pausecolor_ = Color.red;

    Thread thread_;

    boolean canrotate_;

    int speed_ = 100;

    TriggerInterface obj_;

    public RotatingPanel( String title )
    {
	this( title, null, null );
    }

    public RotatingPanel( String title, TriggerInterface obj )
    {
	this( title, obj, null );
    }

    public RotatingPanel( String title,
			  TriggerInterface obj,			  
			  CriticalSection controller )
    {
	super( new BorderLayout() );

	obj_ = obj;

	slider_ = new Scrollbar( Scrollbar.HORIZONTAL, 100, 5, 5, 200  );
	slider_.addAdjustmentListener( new SliderListener( this ) );

	Panel p = new Panel();
	run_ = new Button( "Run" );
        run_.addActionListener( new RunListener( this ) );
	pause_ = new Button( "Pause" );
        pause_.addActionListener( new PauseListener( this ) );
	p.add( run_ );
	p.add( pause_ );

	arcPanel_ = new ArcPanel( title, controller );
	add( arcPanel_, BorderLayout.NORTH );
	add( slider_, BorderLayout.CENTER );
	add( p, BorderLayout.SOUTH );
    }

    public void run()
    {
	for (;;)
	    {
		try
		    {
			synchronized(this)
			    {
				while( !canrotate_ )
				    wait();
				// synchronization lock is released
				// and thread is suspended
			    }

			arcPanel_.rotate();
			if ( obj_ != null )
			    {
				if ( arcPanel_.getAngle() == 360 )
				    {
					// performAction may block and raise
				        // the exception InterruptedException
					obj_.performAction();
				    }
			    }
			Thread.currentThread().sleep( speed_ );
		    }
		catch (InterruptedException e)
		    {
			return;
		    }
	    }
    }

    public void setSpeed( int speed )
    {
	speed_ = speed;
    }

    public int getSpeed()
    {
	return speed_;
    }

    public void setTrigger( TriggerInterface trigger )
    {
	obj_ = trigger;
    }

    public void setRotatorColor( Color c )
    {
	arcPanel_.setArcColor( c );
    }

    public Color getRotatorColor()
    {
	return arcPanel_.getArcColor();
    }

    public void setPauseColor( Color c )
    {
	pausecolor_ = c;
	repaint();
    }

    public Color getPauseColor()
    {
	return pausecolor_;
    }

    public void setZero() throws InterruptedException
    {
	arcPanel_.setZero();
        setInActive();
    }

    public synchronized void setActive()
    {
	try
	    {
		canrotate_ = true;
		run_.setEnabled( false );
		pause_.setEnabled( true );
		arcPanel_.setBackground( runcolor_ );
		arcPanel_.disableChangeOfCriticalSection();
		notifyAll();
	    }
	catch ( InterruptedException e )
	    {
		return;
	    }
    }

    public synchronized void setInActive()
    {
	canrotate_ = false;
        run_.setEnabled( true );
        pause_.setEnabled( false );
        arcPanel_.setBackground( pausecolor_ );
	arcPanel_.enableChangeOfCriticalSection();
    }

    public void start() 
    {
	try
	    {
		setZero();
		slider_.setValue( 100 );
		setSpeed( 100 );
		canrotate_ = false;
		thread_ = new Thread( this );
		thread_.start();
	    }
	catch ( InterruptedException e )
	    {
		System.out.println( "start of rotating panel failed!" );
		return;
	    }
    }

    public void stop() 
    {
	thread_.stop();
    }

    class RunListener implements ActionListener
    {

        RotatingPanel owner_;

        public RunListener( RotatingPanel owner )
        {
            owner_ = owner;
        }

	public void actionPerformed( ActionEvent evt )
	{
            owner_.setActive();
	}

    }

    class PauseListener implements ActionListener
    {

        RotatingPanel owner_;

        public PauseListener( RotatingPanel owner )
        {
            owner_ = owner;
        }


	public void actionPerformed( ActionEvent evt )
	{
            owner_.setInActive();
	}

    }

    class SliderListener implements AdjustmentListener
    {

        RotatingPanel owner_;

        public SliderListener( RotatingPanel owner )
        {
            owner_ = owner;
        }

	public void adjustmentValueChanged( AdjustmentEvent evt )
	{
            owner_.setSpeed( evt.getValue() );
	}

    }

}
